import { CustomResource } from 'aws-cdk-lib';
import { IManagedPolicy, IRole } from 'aws-cdk-lib/aws-iam';
import { IFunction } from 'aws-cdk-lib/aws-lambda';
import { ILogGroup } from 'aws-cdk-lib/aws-logs';
import { IBucket } from 'aws-cdk-lib/aws-s3';
import { Construct } from 'constructs';
import { BaseRedshiftDataAccess, RedshiftDataAccessTargetProps } from './base-redshift-data-access';
import { RedshiftDataProps } from './redshift-data-props';
/**
 * Creates an asynchronous custom resource that handles the execution of SQL using Redshift's Data API. If `vpc` and `vpcSubnets` are passed, this construct would also create the Redshift Data Interface VPC endpoint and configure the custom resource in the same VPC subnet.
 *
 * @example
 * const namespace = new dsf.consumption.RedshiftServerlessNamespace(this, 'RedshiftNamespace', {
 *    name: "default",
 *    dbName: 'defaultdb',
 * });
 *
 * const workgroup = new dsf.consumption.RedshiftServerlessWorkgroup(this, "RedshiftWorkgroup", {
 *    name: "redshift-workgroup",
 *    namespace: namespace,
 * });
 *
 * const rsData = workgroup.accessData('DataApi');
 * rsData.createDbRole("EngineeringRole", "defaultdb", "engineering");
 */
export declare class RedshiftData extends BaseRedshiftDataAccess {
    /**
     * The CloudWatch Log Group for the Redshift Data API submission
     */
    readonly submitLogGroup: ILogGroup;
    /**
     * The Lambda Function for the Redshift Data submission
     */
    readonly submitFunction: IFunction;
    /**
     * The IAM Role for the Redshift Data API execution
     */
    readonly executionRole: IRole;
    /**
     * The CloudWatch Log Group for the Redshift Data API status checks
     */
    readonly statusLogGroup: ILogGroup;
    /**
     * The Lambda Function for the Redshift Data API status checks
     */
    readonly statusFunction: IFunction;
    /**
     * The CloudWatch Log Group for the Redshift Data cleaning up lambda
     */
    readonly cleanUpLogGroup?: ILogGroup;
    /**
     * The Lambda function for the S3 data copy cleaning up lambda
     */
    readonly cleanUpFunction?: IFunction;
    /**
     * The IAM Role for the the S3 data copy cleaning up lambda
     */
    readonly cleanUpRole?: IRole;
    /**
     * The managed IAM policy allowing IAM Role to retrieve tag information
     */
    readonly taggingManagedPolicy: IManagedPolicy;
    /**
     * Contains normalized details of the target Redshift cluster/workgroup for data access
     */
    readonly dataAccessTargetProps: RedshiftDataAccessTargetProps;
    private readonly serviceToken;
    constructor(scope: Construct, id: string, props: RedshiftDataProps);
    /**
     * Runs a custom SQL. Once the custom resource finishes execution, the attribute `Data` contains an attribute `execId` which contains the Redshift Data API execution ID. You can then use this to retrieve execution results via the `GetStatementResult` API.
     * @param id The CDK Construct ID
     * @param databaseName The name of the database to run this command
     * @param sql The sql to run
     * @param deleteSql Optional. The sql to run when this resource gets deleted
     * @returns `CustomResource`
     */
    runCustomSQL(id: string, databaseName: string, sql: string, deleteSql?: string): CustomResource;
    /**
     * Creates a new DB role
     * @param id The CDK Construct ID
     * @param databaseName The name of the database to run this command
     * @param roleName The name of the role to create
     * @returns `CustomResource`
     */
    createDbRole(id: string, databaseName: string, roleName: string): CustomResource;
    /**
     * Grants access to the schema to the DB role
     * @param id The CDK Construct ID
     * @param databaseName The name of the database to run this command
     * @param schema The schema where the tables are located in
     * @param roleName The DB role to grant the permissions to
     * @returns `CustomResource`
     */
    grantDbSchemaToRole(id: string, databaseName: string, schema: string, roleName: string): CustomResource;
    /**
     * Grants read permission on all the tables in the `schema` to the DB role
     * @param databaseName The name of the database to run this command
     * @param schema The schema where the tables are located in
     * @param roleName The DB role to grant the permissions to
     * @returns `CustomResource`
     */
    grantSchemaReadToRole(id: string, databaseName: string, schema: string, roleName: string): CustomResource;
    /**
     * Grants both read and write permissions on all the tables in the `schema` to the DB role
     * @param id The CDK Construct ID
     * @param databaseName The name of the database to run this command
     * @param schema The schema where the tables are located in
     * @param roleName The DB role to grant the permissions to
     * @returns `CustomResource`
     */
    grantDbAllPrivilegesToRole(id: string, databaseName: string, schema: string, roleName: string): CustomResource;
    /**
     * Assigns Redshift DB roles to IAM role vs the `RedshiftDbRoles` tag
     * @param dbRoles List of Redshift DB roles to assign to IAM role
     * @param targetRole The IAM role to assign the Redshift DB roles to
     */
    assignDbRolesToIAMRole(dbRoles: string[], targetRole: IRole): void;
    /**
     * Ingest data from S3 into a Redshift table
     * @param id The CDK Construct ID
     * @param databaseName The name of the database to run this command
     * @param targetTable The target table to load the data into
     * @param sourceBucket The bucket where the source data would be coming from
     * @param sourcePrefix The location inside the bucket where the data would be ingested from
     * @param ingestAdditionalOptions Optional. Additional options to pass to the `COPY` command. For example, `delimiter '|'` or `ignoreheader 1`
     * @param role Optional. The IAM Role to use to access the data in S3. If not provided, it would use the default IAM role configured in the Redshift Namespace
     * @returns
     */
    ingestData(id: string, databaseName: string, targetTable: string, sourceBucket: IBucket, sourcePrefix: string, ingestAdditionalOptions?: string, role?: IRole): CustomResource;
    /**
     * Run the `MERGE` query using simplified mode. This command would do an upsert into the target table.
     * @param id The CDK Construct ID
     * @param databaseName The name of the database to run this command
     * @param sourceTable The source table name. Schema can also be included using the following format: `schemaName.tableName`
     * @param targetTable The target table name. Schema can also be included using the following format: `schemaName.tableName`
     * @param sourceColumnId The column in the source table that's used to determine whether the rows in the `sourceTable` can be matched with rows in the `targetTable`. Default is `id`
     * @param targetColumnId The column in the target table that's used to determine whether the rows in the `sourceTable` can be matched with rows in the `targetTable`. Default is `id`
     * @returns `CustomResource`
     */
    mergeToTargetTable(id: string, databaseName: string, sourceTable: string, targetTable: string, sourceColumnId?: string, targetColumnId?: string): CustomResource;
}
