import { CustomResource, RemovalPolicy } from 'aws-cdk-lib';
import { ISecurityGroup } from 'aws-cdk-lib/aws-ec2';
import { IPrincipal, IRole } from 'aws-cdk-lib/aws-iam';
import { IFunction } from 'aws-cdk-lib/aws-lambda';
import { ILogGroup } from 'aws-cdk-lib/aws-logs';
import { Construct } from 'constructs';
import { KafkaApiProps } from './kafka-api-props';
import { Authentication, Acl, MskTopic } from './msk-utils';
import { TrackedConstruct } from '../../../utils';
/**
 * A construct to create a Kafka API admin client
 * @see https://awslabs.github.io/data-solutions-framework-on-aws/
 *
 * @example
 */
export declare class KafkaApi extends TrackedConstruct {
    /**
     * The IAM Role used by the Custom Resource provider when MSK is using IAM authentication
     */
    readonly mskIamRole?: IRole;
    /**
     * The Cloudwatch Log Group used by the Custom Resource provider when MSK is using IAM authentication
     */
    readonly mskIamLogGroup?: ILogGroup;
    /**
     * The Lambda function used by the Custom Resource provider when MSK is using IAM authentication
     */
    readonly mskIamFunction?: IFunction;
    /**
     * The Security Group used by the Custom Resource provider when MSK is using IAM authentication
     */
    readonly mskIamSecurityGroup?: ISecurityGroup[];
    /**
     * The IAM Role used by the Custom Resource provider when MSK is using mTLS authentication
     */
    readonly mskAclRole?: IRole;
    /**
     * The Cloudwatch Log Group used by the Custom Resource provider when MSK is using mTLS authentication
     */
    readonly mskAclLogGroup?: ILogGroup;
    /**
     * The Lambda function used by the Custom Resource provider when MSK is using mTLS authentication
     */
    readonly mskAclFunction?: IFunction;
    /**
     * The Security Group used by the Custom Resource provider when MSK is using mTLS authentication
     */
    readonly mskAclSecurityGroup?: ISecurityGroup[];
    private readonly mskAclServiceToken?;
    private readonly mskIamServiceToken?;
    private readonly removalPolicy;
    private readonly kafkaClientLogLevel;
    private readonly tlsCertifacateSecret?;
    private readonly clusterArn;
    private readonly clusterType;
    /**
     * Constructs a new instance of the Kafka API construct.
     * @param {Construct} scope the Scope of the CDK Construct
     * @param {string} id the ID of the CDK Construct
     * @param {MskServerlessProps} props
     */
    constructor(scope: Construct, id: string, props: KafkaApiProps);
    /**
     * Creates a ACL in the MSK Cluster
     * @param {string} id the CDK ID of the ACL
     * @param {Acl} aclDefinition the Kafka ACL definition
     * @param {RemovalPolicy} removalPolicy Wether to keep the ACL or delete it when removing the resource from the Stack. @default - RemovalPolicy.RETAIN
     * @param {Authentication} clientAuthentication The authentication used by the Kafka API admin client to create the ACL @default - Authentication.MTLS
     * @returns {CustomResource} The MskAcl custom resource created by the Kafka API admin client
     */
    setAcl(id: string, aclDefinition: Acl, removalPolicy?: RemovalPolicy, clientAuthentication?: Authentication): CustomResource;
    /**
     * Creates a topic in the MSK Cluster
     * @param {string} id the CDK ID for Topic
     * @param {Authentication} clientAuthentication The authentication used by the Kafka API admin client to create the topic
     * @param {MskTopic} topicDefinition the Kafka topic definition
     * @param {RemovalPolicy} removalPolicy Wether to keep the topic or delete it when removing the resource from the Stack. @default - RemovalPolicy.RETAIN
     * @param {boolean} waitForLeaders If set to true, waits until metadata for the new topics doesn't throw LEADER_NOT_AVAILABLE
     * @param {number} timeout The time in ms to wait for a topic to be completely created on the controller node @default - 5000
     * @returns {CustomResource} The MskTopic custom resource created by the Kafka API admin client
     */
    setTopic(id: string, clientAuthentication: Authentication, topicDefinition: MskTopic, removalPolicy?: RemovalPolicy, waitForLeaders?: boolean, timeout?: number): CustomResource;
    /**
     * Grant a principal permissions to produce to a topic
     * @param {string} id the CDK resource ID
     * @param {string} topicName the target topic to grant produce permissions on
     * @param {Authentication} clientAuthentication The authentication mode of the producer
     * @param {IPrincipal | string } principal the principal receiving grant produce permissions
     * @param {string} host the host of the producer. @default - * is used
     * @param {RemovalPolicy} removalPolicy the removal policy to apply to the grant. @default - RETAIN is used
     * @param {Authentication} customResourceAuthentication The authentication used by the Kafka API admin client to create the ACL @default - clientAuthentication (same authentication as the target producer)
     * @returns The MskAcl custom resource for MTLS clientAuthentication. Nothing for IAM clientAuthentication
     */
    grantProduce(id: string, topicName: string, clientAuthentication: Authentication, principal: IPrincipal | string, host?: string, removalPolicy?: RemovalPolicy, customResourceAuthentication?: Authentication): CustomResource | undefined;
    /**
     * Grant a principal permissions to consume from a topic
     * @param {string} id the CDK resource ID
     * @param {string} topicName the target topic to grant consume permissions on
     * @param {Authentication} clientAuthentication The authentication mode of the consumer
     * @param {IPrincipal | string } principal the principal receiveing grant consume permissions
     * @param {string} host the host of the consumer. @default - * is used
     * @param {RemovalPolicy} removalPolicy the removal policy to apply to the grant. @default - RETAIN is used
     * @param {Authentication} customResourceAuthentication The authentication used by the Kafka API admin client to create the ACL @default - clientAuthentication (same authentication as the target producer)
     * @returns The MskAcl custom resource for MTLS clientAuthentication. Nothing for IAM clientAuthentication
     */
    grantConsume(id: string, topicName: string, clientAuthentication: Authentication, principal: IPrincipal | string, host?: string, removalPolicy?: RemovalPolicy, customResourceAuthentication?: Authentication): CustomResource | undefined;
}
