import { CustomResource, RemovalPolicy } from 'aws-cdk-lib';
import { ISecurityGroup, IVpc } from 'aws-cdk-lib/aws-ec2';
import { IPrincipal, PolicyDocument } from 'aws-cdk-lib/aws-iam';
import { CfnClusterPolicy, CfnServerlessCluster } from 'aws-cdk-lib/aws-msk';
import { Construct } from 'constructs';
import { MskServerlessProps } from './msk-serverless-props';
import { MskTopic } from './msk-utils';
import { TrackedConstruct } from '../../../utils';
/**
 * A construct to create an MSK Serverless cluster
 * @see https://awslabs.github.io/data-solutions-framework-on-aws/
 *
 * @example
 *
 * const msk = new dsf.streaming.MskServerless(this, 'cluster');
 */
export declare class MskServerless extends TrackedConstruct {
    readonly cluster: CfnServerlessCluster;
    readonly vpc: IVpc;
    readonly brokerSecurityGroup?: ISecurityGroup;
    readonly clusterName: string;
    readonly lambdaSecurityGroup: ISecurityGroup;
    private readonly removalPolicy;
    private readonly kafkaApi;
    /**
     * Constructs a new instance of the EmrEksCluster construct.
     * @param {Construct} scope the Scope of the CDK Construct
     * @param {string} id the ID of the CDK Construct
     * @param {MskServerlessProps} props
     */
    constructor(scope: Construct, id: string, props?: MskServerlessProps);
    /**
     * Creates a topic in the MSK Serverless
     *
     * @param {string} id the CDK id for the topic
     * @param {MskTopic []} topicDefinition the Kafka topic definition
     * @param {RemovalPolicy} removalPolicy Wether to keep the topic or delete it when removing the resource from the Stack. @default - RemovalPolicy.RETAIN
     * @param {boolean} waitForLeaders Wait until metadata for the new topics doesn't throw LEADER_NOT_AVAILABLE
     * @param {number} timeout The time in ms to wait for a topic to be completely created on the controller node @default - 5000
     * @return the custom resource used to create the topic
     */
    addTopic(id: string, topicDefinition: MskTopic, removalPolicy?: RemovalPolicy, waitForLeaders?: boolean, timeout?: number): CustomResource;
    /**
     * Grant a principal to produce data to a topic
     *
     * @param {string} topicName the name of the topic to grant producer permissions
     * @param {IPrincipal} principal the IAM principal to grand producer permissions
     * @return the custom resource used to grant the producer permissions
     */
    grantProduce(topicName: string, principal: IPrincipal): CustomResource | undefined;
    /**
     * Grant a principal the right to consume data from a topic
     *
     * @param {string} topicName the topic to which the principal can consume data from.
     * @param {IPrincipal} principal the IAM principal to grand the consume action.
     * @return the custom resource used to grant the consumer permissions
     */
    grantConsume(topicName: string, principal: IPrincipal): CustomResource | undefined;
    /**
      * Add a cluster policy
      *
      * @param {PolicyDocument} policy the IAM principal to grand the consume action.
      * @param {string} id the CDK id for the Cluster Policy
      * @return {CfnClusterPolicy}
      */
    addClusterPolicy(policy: PolicyDocument, id: string): CfnClusterPolicy;
}
